@@ -20,6 +20,8 @@
 #              15) Zdenek Mihula private communication (TZ8)
 #              16) Olaf Ulrich private communication
 #              17) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,4922.0.html
+#              18) Thomas Modes private communication (G6)
+#              19) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5533.0.html
 #              JD) Jens Duttke private communication (TZ3,FZ30,FZ50)
 #------------------------------------------------------------------------------
 
@@ -30,7 +32,7 @@ use vars qw($VERSION %leicaLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.71';
+$VERSION = '1.83';
 
 sub ProcessPanasonicType2($$$);
 sub WhiteBalanceConv($;$$);
@@ -210,6 +212,7 @@ my %shootingMode = (
     44 => 'Film Grain', #PH (FZ28)
     45 => 'My Color', #PH (GF1)
     46 => 'Photo Frame', #PH (FS7)
+    48 => 'Movie', #PH (GM1)
     # 49 - seen for FS4 (snow?)
     51 => 'HDR', #12
     55 => 'Handheld Night Shot', #PH (FZ47)
@@ -219,6 +222,30 @@ my %shootingMode = (
     63 => 'Glass Through', #17
     64 => 'HDR', #17
     66 => 'Digital Filter', #PH (GF5 "Impressive Art", "Cross Process", "Color Select", "Star")
+    67 => 'Clear Portrait', #18
+    68 => 'Silky Skin', #18
+    69 => 'Backlit Softness', #18
+    70 => 'Clear in Backlight', #18
+    71 => 'Relaxing Tone', #18
+    72 => "Sweet Child's Face", #18
+    73 => 'Distinct Scenery', #18
+    74 => 'Bright Blue Sky', #18
+    75 => 'Romantic Sunset Glow', #18
+    76 => 'Vivid Sunset Glow', #18
+    77 => 'Glistening Water', #18
+    78 => 'Clear Nightscape', #18
+    79 => 'Cool Night Sky', #18
+    80 => 'Warm Glowing Nightscape', #18
+    81 => 'Artistic Nightscape', #18
+    82 => 'Glittering Illuminations', #18
+    83 => 'Clear Night Portrait', #18
+    84 => 'Soft Image of a Flower', #18
+    85 => 'Appetizing Food', #18
+    86 => 'Cute Desert', #18
+    87 => 'Freeze Animal Motion', #18
+    88 => 'Clear Sports Shot', #18
+    89 => 'Monochrome', #18
+    90 => 'Creative Control', #18
 );
 
 %Image::ExifTool::Panasonic::Main = (
@@ -280,7 +307,9 @@ my %shootingMode = (
             2 => 'Manual',
             4 => 'Auto, Focus button', #4
             5 => 'Auto, Continuous', #4
-            # have seens 6 for GF1 - PH
+            6 => 'AF-S', #18 (also seen for GF1 - PH)
+            7 => 'AF-C', #18
+            8 => 'AF-F', #18 (auto-focus flexible)
         },
     },
     0x0f => [
@@ -315,6 +344,7 @@ my %shootingMode = (
                 '32 2'  => '3-area (center)?', # (DMC-L1 guess)
                 '32 3'  => '3-area (right)?', # (DMC-L1 guess)
                 '64 0'  => 'Face Detect',
+                '128 0' => 'Spot Focusing 2', #18
             },
         },
     ],
@@ -325,7 +355,9 @@ my %shootingMode = (
             2 => 'On, Mode 1',
             3 => 'Off',
             4 => 'On, Mode 2',
-            # GF1 also has a "Mode 3"
+            5 => 'Panning', #18
+            # GF1 also has a "Mode 3" - PH
+            6 => 'On, Mode 3', #PH (GX7, sensor shift?)
         },
     },
     0x1c => {
@@ -408,6 +440,7 @@ my %shootingMode = (
             4 => 'Black & White',
             5 => 'Sepia',
             6 => 'Happy', #PH (FX70) (yes, really. you wouldn't want sad colors now would you?)
+            8 => 'Vivid', #PH (SZ3)
         },
     },
     0x29 => { #JD
@@ -455,6 +488,8 @@ my %shootingMode = (
             1 => 'On', #PH (TZ5) [was "Low/High Quality" from ref 4]
             2 => 'Auto Exposure Bracketing (AEB)', #17
             4 => 'Unlimited', #PH (TZ5)
+            8 => 'White Balance Bracketing', #18
+            17 => 'On (with flash)', #forum5597
         },
     },
     0x2b => { #4
@@ -476,10 +511,13 @@ my %shootingMode = (
                 0 => 'Normal',
                 1 => 'Low',
                 2 => 'High',
-                # 3 - observed with LZ6 and TZ5 in Fireworks mode - PH
+                # 3 - observed with LZ6 and TZ5 in Fireworks mode
+                #     and GX7 in Fantasy/Retro/OldDays/HighKey - PH
+                # 4 - observed in MP4 movie with GM1 (EXIF and 0x39 Contrast "Normal") - PH
                 # 5 - observed with FX01, FX40 and FP8 (EXIF contrast "Normal") - PH
                 6 => 'Medium Low', #PH (FZ18)
                 7 => 'Medium High', #PH (FZ18)
+                # 8 - GX7 in DynamicMonochrome mode
                 13 => 'High Dynamic', #PH (FZ47 in ?)
                 # DMC-LC1 values:
                 0x100 => 'Low',
@@ -489,7 +527,7 @@ my %shootingMode = (
         },{
             Name => 'ContrastMode',
             Condition => '$$self{Model} =~ /^DMC-(GF\d+|G2)$/',
-            Notes => 'these values are used by the G2, GF1, GF2, GF3 and GF5',
+            Notes => 'these values are used by the G2, GF1, GF2, GF3, GF5 and GF6',
             Writable => 'int16u',
             PrintConv => { # (decoded for GF1 unless otherwise noted)
                 0 => '-2',
@@ -498,7 +536,7 @@ my %shootingMode = (
                 3 => '+1',
                 4 => '+2',
                 # Note: Other Contrast tags will be "Normal" in any of these modes:
-                # 5 - seen for Portrait (FX80)
+                5 => 'Normal 2', # 5 - seen for Portrait (FX80) and Normal (GF6)
                 7 => 'Nature (Color Film)', # (GF1,G2; GF3 "Miniature")
                 9 => 'Expressive', #(GF3)
                 12 => 'Smooth (Color Film) or Pure (My Color)', #(GF1,G2 "Smooth Color")
@@ -518,6 +556,16 @@ my %shootingMode = (
                 47 => 'Dynamic Mono', #(GF5)
                 50 => 'Impressive Art', #(GF5)
                 51 => 'Cross Process', #(GF5)
+                107 => 'Expressive 2', #(GF6)
+                122 => 'Dynamic Monochrome', #(GF6)
+                # more new modes for GF6:
+                # ? => 'Old Days',
+                # ? => 'Toy Pop',
+                # ? => 'Bleach Bypass',
+                # ? => 'Fantasy',
+                # ? => 'Star Filter',
+                # ? => 'One Point Color',
+                # ? => 'Sunshine',
             },
         },{
             Name => 'ContrastMode',
@@ -703,7 +751,19 @@ my %shootingMode = (
         Name => 'ColorTempKelvin',
         Format => 'int16u',
     },
-    # 0x45 - int16u: 0
+    0x45 => { #19
+        Name => 'BracketSettings',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'No Bracket',
+            1 => '3 Images, Sequence 0/-/+',
+            2 => '3 Images, Sequence -/0/+',
+            3 => '5 Images, Sequence 0/-/+',
+            4 => '5 Images, Sequence -/0/+',
+            5 => '7 Images, Sequence 0/-/+',
+            6 => '7 Images, Sequence -/0/+',
+        },
+    },
     0x46 => { #PH/JD
         Name => 'WBShiftAB',
         Format => 'int16s',
@@ -725,8 +785,14 @@ my %shootingMode = (
             2 => '2nd',
         },
     },
-    # 0x48 - int16u: 0
-    # 0x49 - int16u: 2
+    0x49 => { #19
+        Name => 'LongExposureNoiseReduction',
+        Writable => 'int16u',
+        PrintConv => {
+            1 => 'Off',
+            2 => 'On'
+        }
+    },
     # 0x4a - int16u: 0
     0x4b => { #PH
         Name => 'PanasonicImageWidth',
@@ -760,18 +826,26 @@ my %shootingMode = (
         Name => 'LensType',
         Writable => 'string',
         ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
+        ValueConvInv => '$val',
     },
     0x52 => { #7 (DMC-L1)
         Name => 'LensSerialNumber',
         Writable => 'string',
         ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
+        ValueConvInv => '$val',
     },
     0x53 => { #7 (DMC-L1)
         Name => 'AccessoryType',
         Writable => 'string',
         ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
+        ValueConvInv => '$val',
+    },
+    0x54 => { #19
+        Name => 'AccessorySerialNumber',
+        Writable => 'string',
+        ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
+        ValueConvInv => '$val',
     },
-    # 0x54 - string[14]: "0000000"
     # 0x55 - int16u: 1
     # 0x57 - int16u: 0
     0x59 => { #PH (FS7)
@@ -802,7 +876,15 @@ my %shootingMode = (
             3 => 'High',
         },
     },
-    # 0x5e,0x5f,0x60 - undef[4]
+    # 0x5e,0x5f - undef[4]
+    0x60 => { #18
+        Name => 'LensFirmwareVersion',
+        Writable => 'undef',
+        Format => 'int8u',
+        Count => 4,
+        PrintConv => '$val=~tr/ /./; $val',
+        PrintConvInv => '$val=~tr/./ /; $val',
+    },
     0x61 => { #PH
         Name => 'FaceRecInfo',
         SubDirectory => {
@@ -882,7 +964,13 @@ my %shootingMode = (
         },
     },
     # 0x71 - undef[128] (maybe text stamp text?)
-    # 0x72,0x73,0x74,0x75,0x76,0x77,0x78: 0
+    0x77 => { #18
+        Name => 'BurstSpeed',
+        Writable => 'int16u',
+        Notes => 'images per second',
+    },
+    # 0x72,0x73,0x74,0x75,0x77,0x78: 0
+    # 0x76: 0, (3 for G6 with HDR on, ref 18)
     0x79 => { #PH (GH2)
         Name => 'IntelligentD-Range',
         Writable => 'int16u',
@@ -894,6 +982,11 @@ my %shootingMode = (
         },
     },
     # 0x7a,0x7b: 0
+    0x7c => { #18
+        Name => 'ClearRetouch',
+        Writable => 'int16u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
     0x86 => { #http://dev.exiv2.org/issues/825
         Name => 'ManometerPressure',
         Writable => 'int16u',
@@ -903,7 +996,7 @@ my %shootingMode = (
         PrintConv => 'sprintf("%.1f kPa",$val)',
         PrintConvInv => '$val=~s/ ?kPa//i; $val',
     },
-    0x0089 => {
+    0x89 => {
         Name => 'PhotoStyle',
         Writable => 'int16u',
         PrintConv => {
@@ -916,6 +1009,120 @@ my %shootingMode = (
             6 => 'Portrait',
         },
     },
+    0x8a => { #18
+        Name => 'ShadingCompensation',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On'
+        }
+    },
+    0x8c => {
+        Name => 'AccelerometerZ',
+        Writable => 'int16u',
+        Format => 'int16s',
+        Notes => 'positive is acceleration upwards',
+    },
+    0x8d => {
+        Name => 'AccelerometerX',
+        Writable => 'int16u',
+        Format => 'int16s',
+        Notes => 'positive is acceleration to the left',
+    },
+    0x8e => {
+        Name => 'AccelerometerY',
+        Writable => 'int16u',
+        Format => 'int16s',
+        Notes => 'positive is acceleration backwards',
+    },
+    0x8f => { #18
+        Name => 'CameraOrientation',
+        Writable => 'int8u',
+        PrintConv => {
+            0 => 'Normal',
+            1 => 'Rotate CW',
+            2 => 'Rotate 180',
+            3 => 'Rotate CCW',
+            4 => 'Tilt Upwards',
+            5 => 'Tilt Downwards'
+        }
+    },
+    0x90 => {
+        Name => 'RollAngle',
+        Writable => 'int16u',
+        Format => 'int16s',
+        Notes => 'degrees of clockwise camera rotation',
+        ValueConv => '$val / 10',
+        ValueConvInv => '$val * 10',
+    },
+    0x91 => {
+        Name => 'PitchAngle',
+        Writable => 'int16u',
+        Format => 'int16s',
+        Notes => 'degrees of upward camera tilt',
+        ValueConv => '-$val / 10',
+        ValueConvInv => '-$val * 10',
+    },
+    0x93 => { #18
+        Name => 'SweepPanoramaDirection',
+        Writable => 'int8u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Left to Right',
+            2 => 'Right to Left',
+            3 => 'Top to Bottom',
+            4 => 'Bottom to Top'
+        }
+    },
+    0x94 => { #18
+        Name => 'SweepPanoramaFieldOfView',
+        Writable => 'int16u'
+    },
+    0x96 => { #18
+        Name => 'TimerRecording',
+        Writable => 'int8u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Time Lapse',
+            2 => 'Stop-motion Animation',
+        },
+    },
+    0x9d => { #18
+        Name => 'InternalNDFilter',
+        Writable => 'rational64u'
+    },
+    0x9e => { #18
+        Name => 'HDR',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            100 => '1 EV',
+            200 => '2 EV',
+            300 => '3 EV',
+            32868 => '1 EV (Auto)',
+            32968 => '2 EV (Auto)',
+            33068 => '3 EV (Auto)',
+        },
+    },
+    0x9f => { #18
+        Name => 'ShutterType',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Mechanical',
+            1 => 'Electronic',
+            2 => 'Hybrid', #PH (GM1, 1st curtain electronic, 2nd curtain mechanical)
+        },
+    },
+    0xa3 => { #18
+        Name => 'ClearRetouchValue',
+        Writable => 'rational64u',
+        # undef if ClearRetouch is off, 0 if it is on
+    },
+    0xab => { #18
+        Name => 'TouchAE',
+        Writable => 'int16u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
     0x0e00 => {
         Name => 'PrintIM',
         Description => 'Print Image Matching',
@@ -1363,7 +1570,7 @@ my %shootingMode = (
         Name => 'ShotInfo',
         SubDirectory => { TagTable => 'Image::ExifTool::Panasonic::ShotInfo' },
     },
-    # 0x0410 - int8u[16]: first byte is FileNumber 
+    # 0x0410 - int8u[16]: first byte is FileNumber
     # 0x0411 - int8u[4]: first number is FilmMode (1=Standard,2=Vivid,3=Natural,4=BW Natural,5=BW High Contrast)
     0x0412 => { Name => 'FilmMode',         Writable => 'string' },
     0x0413 => { Name => 'WB_RGBLevels',     Writable => 'rational64u', Count => 3 },
@@ -1389,11 +1596,14 @@ my %shootingMode = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     GROUPS => { 0 => 'MakerNotes', 1 => 'Leica', 2 => 'Camera' },
-    NOTES => 'This information is written by the S2 (as a trailer in JPEG images).',
+    NOTES => q{
+        This information is written by the S2 and M (Typ 240), as a trailer in JPEG
+        images.
+    },
     0x300 => {
         Name => 'PreviewImage',
         Writable => 'undef',
-        Notes => 'S2',
+        Notes => 'S2 and M (Typ 240)',
         DataTag => 'PreviewImage',
         RawConv => q{
             return \$val if $val =~ /^Binary/;
@@ -1415,6 +1625,7 @@ my %shootingMode = (
         Name => 'LensType',
         Writable => 'string',
         ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
+        ValueConvInv => '$val',
     },
     # 0x340 - same as 0x302
 );
@@ -1567,6 +1778,21 @@ my %shootingMode = (
         Tags extracted from the PANA user data found in MP4 videos from models such
         as the DMC-FT20.
     },
+    0 => {
+        Name => 'ThumbnailTest',
+        Format => 'undef[0x1000]',
+        Hidden => 1,
+        RawConv => q{
+            if (substr($val,0x5c,3) eq "\xff\xd8\xff") {
+                $$self{ThumbType} = 1;
+            } elsif (substr($val,0x546,3) eq "\xff\xd8\xff") {
+                $$self{ThumbType} = 2;
+            } else {
+                $$self{ThumbType} = 0;
+            }
+            return undef;
+        },
+    },
     4 => {
         Name => 'Model',
         Description => 'Camera Model Name',
@@ -1574,18 +1800,45 @@ my %shootingMode = (
     },
     0x58 => {
         Name => 'ThumbnailWidth',
+        Condition => '$$self{ThumbType} == 1',
+        Notes => 'Panasonic models',
         Format => 'int16u',
     },
     0x5a => {
         Name => 'ThumbnailHeight',
+        Condition => '$$self{ThumbType} == 1',
         Format => 'int16u',
     },
     0x5c => {
         Name => 'ThumbnailImage',
+        Condition => '$$self{ThumbType} == 1',
         Format => 'undef[16384]',
-        RawConv => '$val=~/^\xff\xd8\xff/ ? $val : undef',
         ValueConv => '$val=~s/\0*$//; \$val',   # remove trailing zeros
     },
+    # 0x5c - there is some messed-up EXIF-IFD-looking data starting here in
+    #        Leica X VARIO MP4 videos, but it doesn't quite make sense
+    0x536 => { # (Leica X VARIO)
+        Name => 'ThumbnailWidth',
+        Condition => '$$self{ThumbType} == 2',
+        Notes => 'Leica X Vario',
+        Format => 'int32uRev', # (little-endian)
+    },
+    0x53a => { # (Leica X VARIO)
+        Name => 'ThumbnailHeight',
+        Condition => '$$self{ThumbType} == 2',
+        Format => 'int32uRev', # (little-endian)
+    },
+    0x53e => { # (Leica X VARIO)
+        Name => 'ThumbnailLength',
+        Condition => '$$self{ThumbType} == 2',
+        Format => 'int32uRev', # (little-endian)
+    },
+    0x546 => { # (Leica X VARIO)
+        Name => 'ThumbnailImage',
+        Condition => '$$self{ThumbType} == 2',
+        Format => 'undef[$val{0x53e}]',
+        Binary => 1,
+    },
     0x4068 => {
         Name => 'ExifData',
         Condition => '$$valPt =~ /^\xff\xd8\xff\xe1..Exif\0\0/s',
@@ -1603,19 +1856,28 @@ my %shootingMode = (
     AdvancedSceneMode => {
         SeparateTable => 'Panasonic AdvancedSceneMode',    # print values in a separate table
         Require => {
-            0 => 'SceneMode',
-            1 => 'AdvancedSceneType',
+            0 => 'Model',
+            1 => 'SceneMode',
+            2 => 'AdvancedSceneType',
         },
-        ValueConv => '"$val[0] $val[1]"',
+        ValueConv => '"$val[0] $val[1] $val[2]"',
         PrintConv => { #PH
             OTHER => sub {
-                my $val = shift;
+                my ($val,$flag,$conv) = @_;
+                $val =~ s/.* (\d+ \d+)/$1/; # drop model name
+                return $$conv{$val} if $$conv{$val};
                 my @v = split ' ', $val;
                 my $prt = $shootingMode{$v[0]};
-                return $v[1] == 1 ? $prt : "$prt ($v[1])" if $prt;
+                # AdvancedSceneType=1 for non-automatic modes P,A,S,SCN (ref 19)
+                # AdvancedSceneType=5 for automatic mode iA (ref 19)
+                if ($prt) {
+                    return $prt if $v[1] == 1;
+                    return "$prt (intelligent auto)" if $v[1] == 5;
+                    return "$prt ($v[1])";
+                }
                 return "Unknown ($val)";
             },
-            Notes => 'A Composite tag derived from SceneMode and AdvancedSceneType.',
+            Notes => 'A Composite tag derived from Model, SceneMode and AdvancedSceneType.',
             '0 1' => 'Off',
             '2 2' => 'Outdoor Portrait', #(FZ28)
             '2 3' => 'Indoor Portrait', #(FZ28)
@@ -1650,14 +1912,34 @@ my %shootingMode = (
             '59 3' => 'High Key', #(GF5)
             '59 4' => 'Sepia', #(GF3,GF5)
             '59 5' => 'High Dynamic', #(GF3,GF5)
-            '59 6' => 'Minature', #(GF3)
+            '59 6' => 'Miniature', #(GF3)
             '59 9' => 'Low Key', #(GF5)
             '59 10' => 'Toy Effect', #(GF5)
             '59 11' => 'Dynamic Monochrome', #(GF5)
             '59 12' => 'Soft', #(GF5)
+            '66 1' => 'Impressive Art', #19
             '66 2' => 'Cross Process', #(GF5)
-            '66 3' => 'Color Select', #(GF5)
+            '66 3' => 'Color Select', #(GF5) (called "One Point Color" by some other models - PH)
             '66 4' => 'Star', #(GF5)
+            '90 3' => 'Old Days', #18
+            '90 4' => 'Sunshine', #18
+            '90 5' => 'Bleach Bypass', #18
+            '90 6' => 'Toy Pop', #18
+            '90 7' => 'Fantasy', #18
+            '90 8' => 'Monochrome', #PH (GX7)
+            '90 9' => 'Rough Monochrome', #PH (GX7)
+            '90 10' => 'Silky Monochrome', #PH (GX7)
+            # TZ40 Creative Control modes (ref 19)
+            'DMC-TZ40 90 1' => 'Expressive',
+            'DMC-TZ40 90 2' => 'Retro',
+            'DMC-TZ40 90 3' => 'High Key',
+            'DMC-TZ40 90 4' => 'Sepia',
+            'DMC-TZ40 90 5' => 'High Dynamic',
+            'DMC-TZ40 90 6' => 'Miniature',
+            'DMC-TZ40 90 9' => 'Low Key',
+            'DMC-TZ40 90 10' => 'Toy Effect',
+            'DMC-TZ40 90 11' => 'Dynamic Monochrome',
+            'DMC-TZ40 90 12' => 'Soft',
         },
     },
 );
@@ -1712,38 +1994,38 @@ sub WhiteBalanceConv($;$$)
 # - deletes LeicaTrailer member and sets LeicaTrailerPos when successful
 sub ProcessLeicaTrailer($;$)
 {
-    my ($exifTool, $newPos) = @_;
-    my $trail = $$exifTool{LeicaTrailer};
-    my $raf = $$exifTool{RAF};
+    my ($et, $newPos) = @_;
+    my $trail = $$et{LeicaTrailer};
+    my $raf = $$et{RAF};
     my $trailPos = $$trail{TrailPos};
     my $pos = $trailPos || $$trail{Offset};
     my $len = $$trail{TrailLen} || $$trail{Size};
     my ($buff, $result, %tagPtr);
 
-    delete $$exifTool{LeicaTrailer} if $trailPos;   # done after this
+    delete $$et{LeicaTrailer} if $trailPos;   # done after this
     unless ($len > 0) {
-        $exifTool->Warn('Missing Leica MakerNote trailer', 1) if $trailPos;
-        delete $$exifTool{LeicaTrailer};
+        $et->Warn('Missing Leica MakerNote trailer', 1) if $trailPos;
+        delete $$et{LeicaTrailer};
         return undef;
     }
     my $oldPos = $raf->Tell();
     my $ok = ($raf->Seek($pos, 0) and $raf->Read($buff, $len) == $len);
     $raf->Seek($oldPos, 0);
     unless ($ok) {
-        $exifTool->Warn('Error reading Leica MakerNote trailer', 1) if $trailPos;
+        $et->Warn('Error reading Leica MakerNote trailer', 1) if $trailPos;
         return undef;
     }
     # look for Leica MakerNote header (should be at start of
     # trailer, but allow up to 256 bytes of garbage just in case)
     if ($buff !~ /^(.{0,256})LEICA\0..../sg) {
         my $what = $trailPos ? 'trailer' : 'offset';
-        $exifTool->Warn("Invalid Leica MakerNote $what", 1);
+        $et->Warn("Invalid Leica MakerNote $what", 1);
         return undef;
     }
     my $junk = $1;
     my $start = pos($buff) - 10;
     if ($start and not $trailPos) {
-        $exifTool->Warn('Invalid Leica MakerNote offset', 1);
+        $et->Warn('Invalid Leica MakerNote offset', 1);
         return undef;
     }
 #
@@ -1752,17 +2034,17 @@ sub ProcessLeicaTrailer($;$)
     my $hdrLen = 8;
     my $dirStart = $start + $hdrLen;
     my $tagInfo = $$trail{TagInfo};
-    if ($$exifTool{HTML_DUMP}) {
+    if ($$et{HTML_DUMP}) {
         my $name = $$tagInfo{Name};
-        $exifTool->HDump($pos+$start, $len-$start, "$name value", 'Leica MakerNote trailer', 4);
-        $exifTool->HDump($pos+$start, $hdrLen, "MakerNotes header", $name);
-    } elsif ($exifTool->Options('Verbose')) {
+        $et->HDump($pos+$start, $len-$start, "$name value", 'Leica MakerNote trailer', 4);
+        $et->HDump($pos+$start, $hdrLen, "MakerNotes header", $name);
+    } elsif ($et->Options('Verbose')) {
         my $where = sprintf('at offset 0x%x', $pos);
-        $exifTool->VPrint(0, "Leica MakerNote trailer ($len bytes $where):\n");
+        $et->VPrint(0, "Leica MakerNote trailer ($len bytes $where):\n");
     }
     # delete LeicaTrailer member so we don't try to process it again
-    delete $$exifTool{LeicaTrailer};
-    $$exifTool{LeicaTrailerPos} = $pos + $start;    # return actual start position of Leica trailer
+    delete $$et{LeicaTrailer};
+    $$et{LeicaTrailerPos} = $pos + $start;    # return actual start position of Leica trailer
 
     my $oldOrder = GetByteOrder();
     my $num = Get16u(\$buff, $dirStart);            # get entry count
@@ -1773,30 +2055,40 @@ sub ProcessLeicaTrailer($;$)
     my $fix = 0;
     if ($valStart < $len) {
         my $valBlock = Image::ExifTool::MakerNotes::GetValueBlocks(\$buff, $dirStart, \%tagPtr);
-        # find the minimum offset (excluding the PreviewImage tag 0x300)
+        # find the minimum offset (excluding the PreviewImage tag 0x300 and 0x301)
         my $minPtr;
         foreach (keys %tagPtr) {
             my $ptr = $tagPtr{$_};
-            next if $_ == 0x300 or not $ptr;
+            next if $_ == 0x300 or $_ == 0x301 or not $ptr or $ptr == 0xffffffff;
             $minPtr = $ptr if not defined $minPtr or $minPtr > $ptr;
         }
         if ($minPtr) {
             my $diff = $minPtr - ($valStart + $pos);
-            # scan value data for the first non-zero byte
             pos($buff) = $valStart;
-            if ($buff =~ /[^\0]/g) {
-                my $n = pos($buff) - 1 - $valStart; # number of zero bytes
-                # S2 writes 282 bytes of zeros, exiftool writes none
-                my $expect = $n >= 282 ? 282 : 0;
-                my $fixBase = $exifTool->Options('FixBase');
-                if ($diff != $expect or defined $fixBase) {
-                    $fix = $expect - $diff;
-                    if (defined $fixBase) {
-                        $fix = $fixBase if $fixBase ne '';
-                        $exifTool->Warn("Adjusted MakerNotes base by $fix",1);
-                    } else {
-                        $exifTool->Warn("Possibly incorrect maker notes offsets (fixed by $fix)",1);
-                    }
+            my $expect;
+            if ($$et{Model} eq 'S2') {
+                # scan value data for the first non-zero byte
+                if ($buff =~ /[^\0]/g) {
+                    my $n = pos($buff) - 1 - $valStart; # number of zero bytes
+                    # S2 writes 282 bytes of zeros, exiftool writes none
+                    $expect = $n >= 282 ? 282 : 0;
+                }
+            } else { # M (Type 240)
+                # scan for the lens type (M writes 114 bytes of garbage first)
+                if ($buff =~ /\G.{114}([\x20-\x7f]*\0*)/sg and length($1) >= 50) {
+                    $expect = 114;
+                }
+            }
+            my $fixBase = $et->Options('FixBase');
+            if (not defined $expect) {
+                $et->Warn('Unrecognized Leica trailer structure');
+            } elsif ($diff != $expect or defined $fixBase) {
+                $fix = $expect - $diff;
+                if (defined $fixBase) {
+                    $fix = $fixBase if $fixBase ne '';
+                    $et->Warn("Adjusted MakerNotes base by $fix",1);
+                } else {
+                    $et->Warn("Possibly incorrect maker notes offsets (fixed by $fix)",1);
                 }
             }
         }
@@ -1816,12 +2108,16 @@ sub ProcessLeicaTrailer($;$)
     );
     my $tagTablePtr = GetTagTable($$tagInfo{SubDirectory}{TagTable});
     if ($newPos) { # are we writing?
+        if ($$et{Model} ne 'S2') {
+            $et->Warn('Leica MakerNote trailer too messed up to edit.  Copying as a block', 1);
+            return $buff;
+        }
         # set position of new MakerNote IFD (+ 8 for Leica MakerNote header)
         $dirInfo{NewDataPos} = $newPos + $start + 8;
-        $result = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+        $result = $et->WriteDirectory(\%dirInfo, $tagTablePtr);
         # write preview image last if necessary and fix up the preview offsets
-        my $previewInfo = $$exifTool{PREVIEW_INFO};
-        delete $$exifTool{PREVIEW_INFO};
+        my $previewInfo = $$et{PREVIEW_INFO};
+        delete $$et{PREVIEW_INFO};
         if ($result) {
             if ($previewInfo) {
                 my $fixup = $previewInfo->{Fixup};
@@ -1833,23 +2129,23 @@ sub ProcessLeicaTrailer($;$)
         }
     } else {
         # extract information
-        $result = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        $result = $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         # also extract as a block if necessary
-        if ($exifTool->Options('MakerNotes') or
-            $exifTool->{REQ_TAG_LOOKUP}{lc($$tagInfo{Name})})
+        if ($et->Options('MakerNotes') or
+            $$et{REQ_TAG_LOOKUP}{lc($$tagInfo{Name})})
         {
             # makernote header must be included in RebuildMakerNotes call
             $dirInfo{DirStart} -= 8;
             $dirInfo{DirLen} += 8;
-            $exifTool->{MAKER_NOTE_BYTE_ORDER} = GetByteOrder();
-            # rebuild maker notes (creates $exifTool->{MAKER_NOTE_FIXUP})
-            my $val = Image::ExifTool::Exif::RebuildMakerNotes($exifTool, $tagTablePtr, \%dirInfo);
+            $$et{MAKER_NOTE_BYTE_ORDER} = GetByteOrder();
+            # rebuild maker notes (creates $$et{MAKER_NOTE_FIXUP})
+            my $val = Image::ExifTool::Exif::RebuildMakerNotes($et, $tagTablePtr, \%dirInfo);
             unless (defined $val) {
-                $exifTool->Warn('Error rebuilding maker notes (may be corrupt)') if $len > 4;
+                $et->Warn('Error rebuilding maker notes (may be corrupt)') if $len > 4;
                 $val = $buff,
             }
-            my $key = $exifTool->FoundTag($tagInfo, $val);
-            $exifTool->SetGroup($key, 'ExifIFD');
+            my $key = $et->FoundTag($tagInfo, $val);
+            $et->SetGroup($key, 'ExifIFD');
         }
     }
     SetByteOrder($oldOrder);
@@ -1875,7 +2171,7 @@ Panasonic and Leica maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
