@@ -19,6 +19,7 @@
 #               4) http://www.fileformat.info/format/psd/egff.htm
 #               5) http://www.telegraphics.com.au/svn/psdparse/trunk/resources.c
 #               6) http://libpsd.graphest.com/files/Photoshop%20File%20Formats.pdf
+#               7) http://www.adobe.com/devnet-apps/photoshop/fileformatashtml/PhotoshopFileFormats.htm
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Photoshop;
@@ -27,7 +28,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD $iptcDigestInfo);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.41';
+$VERSION = '1.44';
 
 sub ProcessPhotoshop($$$);
 sub WritePhotoshop($$$);
@@ -73,7 +74,7 @@ my %psdMap = (
     0x03f0 => { Unknown => 1, Name => 'PStringCaption' },
     0x03f1 => { Unknown => 1, Name => 'BorderInformation' },
     0x03f2 => { Unknown => 1, Name => 'BackgroundColor' },
-    0x03f3 => { Unknown => 1, Name => 'PrintFlags' },
+    0x03f3 => { Unknown => 1, Name => 'PrintFlags', Format => 'int8u' },
     0x03f4 => { Unknown => 1, Name => 'BW_HalftoningInfo' },
     0x03f5 => { Unknown => 1, Name => 'ColorHalftoningInfo' },
     0x03f6 => { Unknown => 1, Name => 'DuotoneHalftoningInfo' },
@@ -81,14 +82,14 @@ my %psdMap = (
     0x03f8 => { Unknown => 1, Name => 'ColorTransferFuncs' },
     0x03f9 => { Unknown => 1, Name => 'DuotoneTransferFuncs' },
     0x03fa => { Unknown => 1, Name => 'DuotoneImageInfo' },
-    0x03fb => { Unknown => 1, Name => 'EffectiveBW' },
+    0x03fb => { Unknown => 1, Name => 'EffectiveBW', Format => 'int8u' },
     0x03fc => { Unknown => 1, Name => 'ObsoletePhotoshopTag1' },
     0x03fd => { Unknown => 1, Name => 'EPSOptions' },
     0x03fe => { Unknown => 1, Name => 'QuickMaskInfo' },
     0x03ff => { Unknown => 1, Name => 'ObsoletePhotoshopTag2' },
-    0x0400 => { Unknown => 1, Name => 'LayerStateInfo' },
+    0x0400 => { Unknown => 1, Name => 'TargetLayerID', Format => 'int16u' }, # (LayerStateInfo)
     0x0401 => { Unknown => 1, Name => 'WorkingPath' },
-    0x0402 => { Unknown => 1, Name => 'LayersGroupInfo' },
+    0x0402 => { Unknown => 1, Name => 'LayersGroupInfo', Format => 'int16u' },
     0x0403 => { Unknown => 1, Name => 'ObsoletePhotoshopTag3' },
     0x0404 => {
         Name => 'IPTCData',
@@ -143,14 +144,14 @@ my %psdMap = (
             TagTable => 'Image::ExifTool::ICC_Profile::Main',
         },
     },
-    0x0410 => { Unknown => 1, Name => 'Watermark' },
-    0x0411 => { Unknown => 1, Name => 'ICC_Untagged' },
-    0x0412 => { Unknown => 1, Name => 'EffectsVisible' },
+    0x0410 => { Unknown => 1, Name => 'Watermark', Format => 'int8u' },
+    0x0411 => { Unknown => 1, Name => 'ICC_Untagged', Format => 'int8u' },
+    0x0412 => { Unknown => 1, Name => 'EffectsVisible', Format => 'int8u' },
     0x0413 => { Unknown => 1, Name => 'SpotHalftone' },
-    0x0414 => { Unknown => 1, Name => 'IDsBaseValue', Description => 'IDs Base Value' },
+    0x0414 => { Unknown => 1, Name => 'IDsBaseValue', Description => 'IDs Base Value', Format => 'int32u' },
     0x0415 => { Unknown => 1, Name => 'UnicodeAlphaNames' },
-    0x0416 => { Unknown => 1, Name => 'IndexedColourTableCount' },
-    0x0417 => { Unknown => 1, Name => 'TransparentIndex' },
+    0x0416 => { Unknown => 1, Name => 'IndexedColourTableCount', Format => 'int16u' },
+    0x0417 => { Unknown => 1, Name => 'TransparentIndex', Format => 'int16u' },
     0x0419 => {
         Name => 'GlobalAltitude',
         Writable => 'int32u',
@@ -183,9 +184,12 @@ my %psdMap = (
         Writable => 'string',
         Protected => 1,
         Notes => q{
-            when writing, special values of "new" and "old" represent the digests of the
-            IPTC from the edited and original files respectively, and are undefined if
-            the IPTC does not exist in the respective file
+            this tag indicates provides a way for XMP-aware applications to indicate
+            that the XMP is synchronized with the IPTC.  When writing, special values of
+            "new" and "old" represent the digests of the IPTC from the edited and
+            original files respectively, and are undefined if the IPTC does not exist in
+            the respective file.  Set this to "new" as an indication that the XMP is
+            synchronized with the IPTC
         },
         # also note the 'new' feature requires that the IPTC comes before this tag is written
         ValueConv => 'unpack("H*", $val)',
@@ -193,7 +197,7 @@ my %psdMap = (
             if (lc($val) eq 'new' or lc($val) eq 'old') {
                 {
                     local $SIG{'__WARN__'} = sub { };
-                    return lc($val) if eval 'require Digest::MD5';
+                    return lc($val) if eval { require Digest::MD5 };
                 }
                 warn "Digest::MD5 must be installed\n";
                 return undef;
@@ -208,6 +212,32 @@ my %psdMap = (
     0x0429 => { Unknown => 1, Name => 'LayerComps' }, #5
     0x042a => { Unknown => 1, Name => 'AlternateDuotoneColors' }, #5
     0x042b => { Unknown => 1, Name => 'AlternateSpotColors' }, #5
+    0x042d => { #7
+        Name => 'LayerSelectionIDs',
+        Description => 'Layer Selection IDs',
+        Unknown => 1,
+        ValueConv => q{
+            my ($n, @a) = unpack("nN*",$val);
+            $#a = $n - 1 if $n > @a;
+            return join(' ', @a);
+        },
+    },
+    0x042e => { Unknown => 1, Name => 'HDRToningInfo' }, #7
+    0x042f => { Unknown => 1, Name => 'PrintInfo' }, #7
+    0x0430 => { Unknown => 1, Name => 'LayerGroupsEnabledID', Format => 'int8u' }, #7
+    0x0431 => { Unknown => 1, Name => 'ColorSamplersResource2' }, #7
+    0x0432 => { Unknown => 1, Name => 'MeasurementScale' }, #7
+    0x0433 => { Unknown => 1, Name => 'TimelineInfo' }, #7
+    0x0434 => { Unknown => 1, Name => 'SheetDisclosure' }, #7
+    0x0435 => { Unknown => 1, Name => 'DisplayInfo' }, #7
+    0x0436 => { Unknown => 1, Name => 'OnionSkins' }, #7
+    0x0438 => { Unknown => 1, Name => 'CountInfo' }, #7
+    0x043a => { Unknown => 1, Name => 'PrintInfo2' }, #7
+    0x043b => { Unknown => 1, Name => 'PrintStyle' }, #7
+    0x043c => { Unknown => 1, Name => 'MacintoshNSPrintInfo' }, #7
+    0x043d => { Unknown => 1, Name => 'WindowsDEVMODE' }, #7
+    0x043e => { Unknown => 1, Name => 'AutoSaveFilePath' }, #7
+    0x043f => { Unknown => 1, Name => 'AutoSaveFormat' }, #7
     # 0x07d0-0x0bb6 Path information
     0x0bb7 => {
         Name => 'ClippingPathName',
@@ -218,6 +248,9 @@ my %psdMap = (
             return Image::ExifTool::Photoshop::ConvertPascalString($self,$val);
         },
     },
+    # 0x0fa0-0x1387 - plug-in resources (ref 7)
+    0x1b58 => { Unknown => 1, Name => 'ImageReadyVariables' }, #7
+    0x1f40 => { Unknown => 1, Name => 'LightroomWorkflow' }, #7
     0x2710 => { Unknown => 1, Name => 'PrintFlagsInfo' },
 );
 
@@ -344,7 +377,7 @@ sub AUTOLOAD
 # Returns: Strings, concatenated with ', '
 sub ConvertPascalString($$)
 {
-    my ($exifTool, $inStr) = @_;
+    my ($et, $inStr) = @_;
     my $outStr = '';
     my $len = length($inStr);
     my $i=0;
@@ -355,8 +388,8 @@ sub ConvertPascalString($$)
         $outStr .= substr($inStr, $i+1, $n);
         $i += $n + 1;
     }
-    my $charset = $exifTool->Options('CharsetPhotoshop') || 'Latin';
-    return $exifTool->Decode($outStr, $charset);
+    my $charset = $et->Options('CharsetPhotoshop') || 'Latin';
+    return $et->Decode($outStr, $charset);
 }
 
 #------------------------------------------------------------------------------
@@ -366,15 +399,15 @@ sub ConvertPascalString($$)
 # Returns: 1 on success
 sub ProcessPhotoshop($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart};
     my $dirEnd = $pos + $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $success = 0;
 
     SetByteOrder('MM');     # Photoshop is always big-endian
-    $verbose and $exifTool->VerboseDir('Photoshop', 0, $$dirInfo{DirLen});
+    $verbose and $et->VerboseDir('Photoshop', 0, $$dirInfo{DirLen});
 
     # scan through resource blocks:
     # Format: 0) Type, 4 bytes - '8BIM' (or the rare 'PHUT', 'DCSR' or 'AgHg')
@@ -391,7 +424,7 @@ sub ProcessPhotoshop($$$)
             $ttPtr = GetTagTable('Image::ExifTool::Photoshop::Unknown');
         } else {
             $type =~ s/([^\w])/sprintf("\\x%.2x",ord($1))/ge;
-            $exifTool->Warn(qq{Bad Photoshop IRB resource "$type"});
+            $et->Warn(qq{Bad Photoshop IRB resource "$type"});
             last;
         }
         my $tag = Get16u($dataPt, $pos + 4);
@@ -402,13 +435,13 @@ sub ProcessPhotoshop($$$)
         $pos += $nameLen;
         ++$pos unless $nameLen & 0x01;
         if ($pos + 4 > $dirEnd) {
-            $exifTool->Warn("Bad Photoshop resource block");
+            $et->Warn("Bad Photoshop resource block");
             last;
         }
         my $size = Get32u($dataPt, $pos);
         $pos += 4;
         if ($size + $pos > $dirEnd) {
-            $exifTool->Warn("Bad Photoshop resource data size $size");
+            $et->Warn("Bad Photoshop resource data size $size");
             last;
         }
         $success = 1;
@@ -418,14 +451,14 @@ sub ProcessPhotoshop($$$)
         } else {
             $name = '';
         }
-        my $tagInfo = $exifTool->GetTagInfo($ttPtr, $tag);
+        my $tagInfo = $et->GetTagInfo($ttPtr, $tag);
         # append resource name to value if requested (braced by "/#...#/")
         if ($tagInfo and defined $$tagInfo{SetResourceName} and
             $$tagInfo{SetResourceName} eq '1' and $name !~ m{/#})
         {
             $val = substr($$dataPt, $pos, $size) . '/#' . $name . '#/';
         }
-        $exifTool->HandleTag($ttPtr, $tag, $val,
+        $et->HandleTag($ttPtr, $tag, $val,
             TagInfo => $tagInfo,
             Extra   => $extra,
             DataPt  => $dataPt,
@@ -446,7 +479,7 @@ sub ProcessPhotoshop($$$)
 # Returns: 1 if this was a valid PSD file, -1 on write error
 sub ProcessPSD($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
     my ($data, $err, $tagTablePtr);
@@ -454,7 +487,7 @@ sub ProcessPSD($$)
     $raf->Read($data, 30) == 30 or return 0;
     $data =~ /^8BPS\0([\x01\x02])/ or return 0;
     SetByteOrder('MM');
-    $exifTool->SetFileType($1 eq "\x01" ? 'PSD' : 'PSB'); # set the FileType tag
+    $et->SetFileType($1 eq "\x01" ? 'PSD' : 'PSB'); # set the FileType tag
     my %dirInfo = (
         DataPt => \$data,
         DirStart => 0,
@@ -466,12 +499,12 @@ sub ProcessPSD($$)
         $raf->Read($data, $len) == $len or return -1;
         Write($outfile, $data) or $err = 1; # write color mode data
         # initialize map of where things are written
-        $exifTool->InitWriteDirs(\%psdMap);
+        $et->InitWriteDirs(\%psdMap);
     } else {
         # process the header
         $tagTablePtr = GetTagTable('Image::ExifTool::Photoshop::Header');
         $dirInfo{DirLen} = 30;
-        $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         $raf->Seek($len, 1) or $err = 1;    # skip over color mode data
     }
     $raf->Read($data, 4) == 4 or $err = 1;
@@ -482,7 +515,7 @@ sub ProcessPSD($$)
     my $rtnVal = 1;
     if ($outfile) {
         # rewrite IRB resources
-        $data = WritePhotoshop($exifTool, \%dirInfo, $tagTablePtr);
+        $data = WritePhotoshop($et, \%dirInfo, $tagTablePtr);
         if ($data) {
             $len = Set32u(length $data);
             Write($outfile, $len, $data) or $err = 1;
@@ -492,7 +525,7 @@ sub ProcessPSD($$)
                 my $tbuf = '';
                 $$trailInfo{OutFile} = \$tbuf;  # rewrite trailer(s)
                 # rewrite all trailers to buffer
-                if ($exifTool->ProcessTrailers($trailInfo)) {
+                if ($et->ProcessTrailers($trailInfo)) {
                     my $copyBytes = $$trailInfo{DataPos} - $raf->Tell();
                     if ($copyBytes >= 0) {
                         # copy remaining PSD file up to start of trailer
@@ -503,9 +536,9 @@ sub ProcessPSD($$)
                             $copyBytes -= $n;
                         }
                         # write the trailer (or not)
-                        $exifTool->WriteTrailerBuffer($trailInfo, $outfile) or $err = 1;
+                        $et->WriteTrailerBuffer($trailInfo, $outfile) or $err = 1;
                     } else {
-                        $exifTool->Warn('Overlapping trailer');
+                        $et->Warn('Overlapping trailer');
                         undef $trailInfo;
                     }
                 } else {
@@ -523,12 +556,12 @@ sub ProcessPSD($$)
         }
         $rtnVal = -1 if $err;
     } elsif ($err) {
-        $exifTool->Warn('File format error');
+        $et->Warn('File format error');
     } else {
-        ProcessPhotoshop($exifTool, \%dirInfo, $tagTablePtr);
+        ProcessPhotoshop($et, \%dirInfo, $tagTablePtr);
         # process trailers if they exist
         my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
-        $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
+        $et->ProcessTrailers($trailInfo) if $trailInfo;
     }
     return $rtnVal;
 }
@@ -566,7 +599,7 @@ be preserved when copying Photoshop information via user-defined tags.
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -583,6 +616,8 @@ under the same terms as Perl itself.
 
 =item L<http://libpsd.graphest.com/files/Photoshop%20File%20Formats.pdf>
 
+=item L<http://www.adobe.com/devnet-apps/photoshop/fileformatashtml/PhotoshopFileFormats.htm>
+
 =back
 
 =head1 SEE ALSO
